@@ -328,8 +328,7 @@ create_session(Request0, Request, SessionID) :-
            [Cookie, SessionID, Path]),
     Request = [session(SessionID)|Request0],
     peer(Request0, Peer),
-    open_session(SessionID, Peer),
-    b_setval(http_session_id, SessionID).
+    open_session(SessionID, Peer).
 
 
 %!  http_open_session(-SessionID, +Options) is det.
@@ -397,6 +396,7 @@ open_session(SessionID, Peer) :-
     get_time(Now),
     assert(current_session(SessionID, Peer)),
     assert(last_used(SessionID, Now)),
+    b_setval(http_session_id, SessionID),
     broadcast(http_session(begin(SessionID, Peer))).
 
 
@@ -591,6 +591,7 @@ expire_session_cookie.
 
 in_header_state :-
     current_output(CGI),
+    is_cgi_stream(CGI),
     cgi_property(CGI, state(header)),
     !.
 
